package ViewAbstraction

import scala.collection.mutable.ArrayBuffer

class CSPFileParser(fname: String){
  /** Tag used to tag annotation in file. */
  private val tag = "@VA"

  /** Does c represent white space? */
  private def whiteSpace(c: Char) = c == ' ' || c == '\t' || c == '\n'

  /** Numbers of symmetric types. */
  // var numTypes = -1

  // Strings extracted from annotations in the CSP file
  val componentProcessNames = new ArrayBuffer[String]
  val componentAlphabets = new ArrayBuffer[String]
  // The indexing types for components; note, these are currently assumed to
  // be distinct
  val identityTypes = new ArrayBuffer[String]
  var serversName = ""

  init()

  /** Parse annotations in file, extracting variables. */
  private def init() = {
    val source = scala.io.Source.fromFile(fname)
    val lines = source.getLines.toList
    for(line <- lines){
      val start = line.indexOfSlice(tag)
      if(start >= 0){
        val rest = line.drop(start+tag.length).dropWhile(whiteSpace)
        val eqIndex = rest.indexOf('=')
        if(eqIndex < 0){ println("Illegal annotation: "+line); sys.exit }
        val lhs0 = rest.take(eqIndex); val lhs = lhs0.filter(!whiteSpace(_))
        val rhs = rest.drop(eqIndex+1).filter(!whiteSpace(_))
        lhs.filter(_.isLetter) match{
          case "componentprocess" => componentProcessNames += rhs
          case "componentalphabet" => componentAlphabets += rhs
          case "componentidentitytype" =>
            identityTypes += rhs
          case "servers" => serversName = rhs
          case _ =>
            println("Annotation variable not recognised: "+lhs0); sys.exit
        }
      }
    } // end of for loop

    // Check all necessary variables defined. 
    if(serversName.isEmpty){
      println("Missing definition for \"servers\""); sys.exit
    }
    val numFamilies = componentProcessNames.length
    if(componentAlphabets.length != numFamilies ||
         identityTypes.length != numFamilies){
      println("Inconsistent numbers of definitions for \"component process\",\n"+
                "\"component alphabet\" and \"identity type\"")
      sys.exit
    }
    if(numFamilies == 0){
      println("Missing definitions for \"component process\",\n"+
                "\"component alphabet\" and \"identity type\"")
      sys.exit
    }
    // if(identityTypes.distinct.length != identityTypes.length){
    //   println("Identity types are not distinct"); sys.exit
    // }
    // numTypes = identityTypes.length
    setNumTypes(identityTypes.distinct.length, identityTypes.length)
  }


}
