import ox.gavin.experiments.{Experiments,Graphs}

object Experiment{
  /** Number of measurements per case. */
  var reps = 5
 
  /* Get hostname */
  val ip = java.net.InetAddress.getLocalHost
  val hostname = ip.getHostName
  println(s"hostname = $hostname")

  val classpath = 
    if(hostname.startsWith("clpc417")) 
      ".:/users/gavinl/Scala/Util:/users/gavinl/bin/FDR4/fdr/lib/fdr.jar"
    else if(hostname.startsWith("aragorn")) 
      ".:/home/gavinl/CADS:/home/gavinl/bin/FDR4/fdr/lib/fdr.jar"
    else{ println("Unknown host"); sys.exit }

  def cmd0 = "scala -cp "+classpath+" ViewAbstraction.VA --timing "

  /** The separate cases to check. */
  val cases = Array(
    "CSP/lockBasedStack.csp --shape 2 0 --shape 1 1",
    "CSP/lockBasedQueue.csp --shape 2 0 --shape 1 1",
    "CSP/MEPeers.csp -k 2",
    "CSP/MEPeersLive.csp -k 2 --checkDeadlock --significancePath 0", 
    "CSP/lockBasedStack.csp -k 2",
    "CSP/lockBasedQueue.csp -k 2",
    //
    "CSP/TreiberStack1.csp  -k 3",
    "CSP/TreiberStack.csp  --shape 3 0 --shape 2 1 --shape 1 2",
    //
    "CSP/lockFreeQueue.csp --shape 2 1 --shape 1 2", 
    "CSP/lockFreeQueue.csp --shape 3 0 --shape 2 1 --shape 1 2",
    // 
    "CSP/listSet1.csp -k 2",
    "CSP/listSet2.csp --shape 4 0 --shape 3 1 --shape 2 2",
    // listSetDI
    "CSP/ManyOne.csp -k 2",
    "CSP/ManyOneLive.csp --shape 2 2 --significancePath 0 1 --significancePath 1 --checkDeadlock"
  )

  val N = cases.length

  val Million = 1000000
  val Billion = 1000000000


  /** A single measurement of cmd, giving time in nanos. */
  def measure(cmd: String) : Double = {
    val time = Experiments.timeProc(cmd, false)
    print(time/Million+" ") // in millis
    time.toDouble
  }

  /** Run the experiments. */
  def run = {
    val results = new Array[(Double,Double)](N)
    val params = new Experiments.Params(reps, reps, 0.05, 0.01)
    for(i <- 0 until N){
      val cmd = cmd0+cases(i); println(cmd)
      val (m,s) = Experiments.iterateMeasurement(measure(cmd), params)
      results(i) = (m,s)
      println; println(m/1000000,s/1000000); println // in millis
    }

    // Print results
    for(i <- 0 until N){
      print(cases(i)+": ")
      val (m,s) = results(i)
      if(m < Billion) println(s"${m/Million} ms +- ${s/Million} ms")
      else println(s"${m/Billion} s +- ${s/Billion} s")
    }
  }

  def main(args: Array[String]) = {
    // parse arguments
    var i = 0
    while(i < args.length) args(i) match{
      case "--reps" => reps = args(i+1).toInt; i += 2
      case arg => println("Unrecognised argument "+arg); sys.exit
    }

    run
  }
}
