import scala.collection.mutable.{Stack,Map,Set}
import java.lang.reflect.Modifier
import ViewAbstraction._

class ScalaInstrumentation{
  def getSize(obj: Object) =
    ox.gavin.profiling.InstrumentationAgent.getObjectSize(obj)

  /** The maximum depth to which to print information. */
  private var maxPrint = 0

  /** The maximum number of entries of each array to print. */
  private var maxArrayPrint = 0

  /** Objects seen so far.  
    *  Each object seen so far maps to the value 1. */
  private var seen: java.util.IdentityHashMap[Object,Int] = null
  // IMPROVE: use a concurrent Set

  /** Add obj to the seen set.  Return true if it wasn't there already. */
  private def add(obj: Object): Boolean = 
    seen.synchronized{seen.put(obj, 1)} != 1

  /** Traverse obj with name name. 
    * @param ignore list of class names to ignore.
    * @param newSearch should this be considered a new search, so results of
    * previous searches are removed? 
    * @param maxP the maximum depth at which to print.
    * @param maxArrayP the maximum number of entries in an array to print
    */
  def traverse(name: String, obj: Object, ignore: List[String], 
               newSearch: Boolean, maxP: Int = 0, maxArrayP: Int = 2): Long = {
    maxPrint = maxP; maxArrayPrint = maxArrayP
    if(newSearch) 
      seen = new java.util.IdentityHashMap[Object,Int]() // Set[Object]()
    if(obj != null) traverse1(name, obj, ignore, 0)
    else{ print(s"$name: null"); 0 }
  }

  /** Recursively traverse obj, with name name, printing out details of memory
    * usage.  Objects in seen are ignored. 
    * @param ignore list of class names to ignore.
    * @param indent the amount of indentation to use.  */
  def traverse1(name: String, obj: Object, ignore: List[String], indent: Int)
      : Long = {
    // val stack = new Stack[(String,Object)]; stack.push(("root",obj0))
    val theIndent = ". "*indent; val theClass = obj.getClass
    val className0 = theClass.toString.split(Array('.')).last
    var total = 0L
    if(indent <= maxPrint) 
      print(theIndent + s"$name: ${obj.toString.take(140)} [$theClass; $className0]") 

    if(!ignore.contains(className0) && /*seen.*/add(obj)){
      val theSize = getSize(obj); total += theSize
      if(indent <= maxPrint) println(s" -> $theSize")

      theClass match{
        case a if a.isArray && a.getName.length != 2 =>
          // primitive type arrays has length two, skip them (they included in
          // the shallow size)
          def processField(i: Int) = {
            val nextObj = java.lang.reflect.Array.get(obj, i)
            if(nextObj != null){
              val c = traverse1(s"$name($i)", nextObj, ignore, 
                                if(i < maxArrayPrint) indent+1 else maxPrint+1)
              synchronized{ total += c }
            }
          }
          val length = java.lang.reflect.Array.getLength(obj)
          if(indent <= maxPrint) println(theIndent+"length "+length)
          if(indent < maxPrint) for(i <- 0 until length) processField(i)
          else for(i <- (0 until length).par) processField(i)
        case c =>
          var currentClass: Class[_] = c
          do {
            def processField(field: java.lang.reflect.Field) = {
              // println(field.getName)
              if (!Modifier.isStatic(field.getModifiers) &&
                    !field.getType.isPrimitive
              ) {
                field.setAccessible(true)
                val tempObject = field.get(obj)
                if (tempObject != null){
                  val c = traverse1(field.getName, tempObject, ignore, indent+1)
                  synchronized{ total += c }
                }
              }
            } // end of processField
            // println(theIndent+currentClass)
            val fields =   currentClass.getDeclaredFields
            if(indent < maxPrint) for(field <- fields) processField(field)
            else for(field <- fields.par) processField(field)
            currentClass = currentClass.getSuperclass
          } while (currentClass != null) ;
      } // end of match
    } // end of if
    else if(indent <= maxPrint) println("(ignored)")
    if(indent <= maxPrint) println(theIndent+total)
    total
  }

  // -------------------------------------------------------

  /** Main function. */ 
  def apply() = {
    val fname = "../CSP/TreiberStack.csp"; val bound = 2
    //val fname = "../CSP/lockFreeQueue.csp"; val bound = 2
    val aShapes = List(Array(1,2), Array(2,1), Array(3,0)); val k = 3

    //val fname = "../CSP/listSetDI.csp"; val bound = 26
    // val fname = "../CSP/listSet2.csp"; val bound = 24
    // val aShapes = List(Array(2,2), Array(3,1), Array(4,0)); val k = 4

    VA.check(fname, k, aShapes, bound = bound, false, List(), false)

    traverse("State", State, List(), true) // ~260K
    traverse("ServerStates", ServerStates, List(), false) // **
    traverse("system", VA.system, List(), false, 2) // ~7.5M
    val checker = VA.checker
//if(false){
    traverse("sysConcViews", checker.sysConcViews, List(), false) // ~360M
    traverse("sysAbsViews", checker.sysAbsViews, List(), false, 8) // ~31M
    traverse("nve1", checker.nve1, List("CanonicalSystemViewSet"), false) // ~6M
    if(checker.nve2 != null) 
      traverse("nve2", checker.nve2, List("CanonicalSystemViewSet"), false) //
    traverse("checker", checker, List(), false, 1)
    if(seen != null) println(s"seen.size = ${seen.size}")
//}

    for(v <- checker.sysConcViews.toList.take(2))
      traverse("view", v, List("State","ServerStates"), true, 3)

  }

}

// import java.lang.instrument._

// object SInstrumentationAgent{
//   private var globalInstrumentation: instrument.Instrumentation = null

//   def premain(agentArgs: String, inst: instrument.Instrumentation) = {
//     globalInstrumentation = inst
//   }

//   def getObjectSize(obj: Object): Long = {
//     if (globalInstrumentation == null) {
//       throw new IllegalStateException("Agent not initialized.")
//     }
//     else globalInstrumentation.getObjectSize(obj)
//   }



// }
